/* MD5C.C - RSA Data Security, Inc., MD5 message-digest algorithm
 */

/* Copyright (C) 1991-2, RSA Data Security, Inc. Created 1991. All
   rights reserved.

   License to copy and use this software is granted provided that it
   is identified as the "RSA Data Security, Inc. MD5 Message-Digest
   Algorithm" in all material mentioning or referencing this software
   or this function.

   License is also granted to make and use derivative works provided
   that such works are identified as "derived from the RSA Data
   Security, Inc. MD5 Message-Digest Algorithm" in all material
   mentioning or referencing the derived work.  
                                                                    
   RSA Data Security, Inc. makes no representations concerning either
   the merchantability of this software or the suitability of this
   software for any particular purpose. It is provided "as is"
   without express or implied warranty of any kind.  
                                                                    
   These notices must be retained in any copies of any part of this
   documentation and/or software.  
 */

#ifndef MD5_C
#define MD5_C

#include <stdio.h>

#include "md5.h"

/* Constants for md5_transform routine.
 */
#define S11 7
#define S12 12
#define S13 17
#define S14 22
#define S21 5
#define S22 9
#define S23 14
#define S24 20
#define S31 4
#define S32 11
#define S33 16
#define S34 23
#define S41 6
#define S42 10
#define S43 15
#define S44 21

#define ENCODE_DECODE 0

void md5_transform(uns32 [4], uns8 [64]);
#if ENCODE_DECODE
void Encode(uns8 *, uns32 *, uns32);
void Decode(uns32 *, uns8 *, uns32);
#endif

/* F, G, H and I are basic MD5 functions. */
#define F(x, y, z) (((x) & (y)) | ((~x) & (z)))
#define G(x, y, z) (((x) & (z)) | ((y) & (~z)))
#define H(x, y, z) ((x) ^ (y) ^ (z))
#define I(x, y, z) ((y) ^ ((x) | (~z)))

/* FF, GG, HH, and II transformations for rounds 1, 2, 3, and 4.
   Rotation is separate from addition to prevent recomputation.
 */
#define FF(a, b, c, d, x, s, ac) { \
  (a) += F ((b), (c), (d)) + (x) + (uns32)(ac); \
  (a) = ROLuns32((a), (s)); \
  (a) += (b); \
}
#define GG(a, b, c, d, x, s, ac) { \
  (a) += G ((b), (c), (d)) + (x) + (uns32)(ac); \
  (a) = ROLuns32((a), (s)); \
  (a) += (b); \
}
#define HH(a, b, c, d, x, s, ac) { \
  (a) += H ((b), (c), (d)) + (x) + (uns32)(ac); \
  (a) = ROLuns32((a), (s)); \
  (a) += (b); \
}
#define II(a, b, c, d, x, s, ac) { \
  (a) += I ((b), (c), (d)) + (x) + (uns32)(ac); \
  (a) = ROLuns32((a), (s)); \
  (a) += (b); \
}

/* MD5 initialization. Begins an MD5 operation, writing a new context.
 */
void md5_init(md5_context *context) {
  context->count[0] = context->count[1] = 0;

  /* Load magic initialization constants.
   */
  context->state[0] = 0x67452301;
  context->state[1] = 0xefcdab89;
  context->state[2] = 0x98badcfe;
  context->state[3] = 0x10325476;
}

/* MD5 block update operation. Continues an MD5 message-digest
     operation, processing another message block, and updating the
     context.
 */
void md5_update(md5_context *context, uns8 *input, uns32 inputLen) {
  uns32 i, index, partLen;

  /* Compute number of bytes mod 64 */
  index = (uns32)((context->count[0] >> 3) & 0x3F);
  /* Update number of bits */
  if ((context->count[0] += ((uns32)inputLen << 3)) < ((uns32)inputLen << 3))
    context->count[1]++;
  context->count[1] += ((uns32)inputLen >> 29);
  
  partLen = 64 - index;
  
  /* Transform as many times as possible.
   */
  if (inputLen >= partLen) {
    memcpy((uns8*)&context->buffer[index], (uns8*)input, partLen);
    md5_transform (context->state, context->buffer);
  
    for (i=partLen; i+63<inputLen; i+=64)
      md5_transform(context->state, &input[i]);
    
    index = 0;
  }
  else
    i = 0;
  
  /* Buffer remaining input */
  memcpy((uns8*)&context->buffer[index], (uns8*)&input[i], inputLen-i);
}

/* MD5 finalization. Ends an MD5 message-digest operation, writing the
     the message digest and zeroizing the context.
 */
void md5_final(uns8 *digest, md5_context *context) {
  uns8 bits[8];
  uns32 index, padLen, t;

  /* Save number of bits */
#if ENCODE_DECODE
  Decode(bits, context->count, 8);
#else
  t = context->count[0];
  bits[0]  = t & 0xff; t >>= 8;
  bits[1]  = t & 0xff; t >>= 8;
  bits[2]  = t & 0xff; t >>= 8;
  bits[3]  = t & 0xff;
  t = context->count[1];
  bits[4]  = t & 0xff; t >>= 8;
  bits[5]  = t & 0xff; t >>= 8;
  bits[6]  = t & 0xff; t >>= 8;
  bits[7]  = t & 0xff; t >>= 8;
#endif


  /* Pad out to 56 mod 64.
   */

  index = (uns32)((context->count[0] >> 3) & 0x3f);
  padLen = (index < 56) ? (56 - index) : (120 - index);
  md5_update(context, (uns8*)hashcom_PADDING, padLen);
  
  /* Append length (before padding) */
  md5_update(context, bits, 8);

  /* Store state in digest */
#if ENCODE_DECODE
  Encode(digest, context->state, 16);
#else
  t = context->state[0];
  digest[0]  = t & 0xff; t >>= 8;
  digest[1]  = t & 0xff; t >>= 8;
  digest[2]  = t & 0xff; t >>= 8;
  digest[3]  = t & 0xff;
  t = context->state[1];
  digest[4]  = t & 0xff; t >>= 8;
  digest[5]  = t & 0xff; t >>= 8;
  digest[6]  = t & 0xff; t >>= 8;
  digest[7]  = t & 0xff; t >>= 8;
  t = context->state[2];
  digest[8]  = t & 0xff; t >>= 8;
  digest[9]  = t & 0xff; t >>= 8;
  digest[10] = t & 0xff; t >>= 8;
  digest[11] = t & 0xff;
  t = context->state[3];
  digest[12] = t & 0xff; t >>= 8;
  digest[13] = t & 0xff; t >>= 8;
  digest[14] = t & 0xff; t >>= 8;
  digest[15] = t & 0xff; t >>= 8;
#endif
  
  /* Zeroize sensitive information.
   */
  memset((uns8*)context, 0, sizeof(md5_context));
}

/* MD5 basic transformation. Transforms state based on block.
 */
void md5_transform(uns32 *state, uns8 *block) {
  uns32 a = state[0],
        b = state[1],
        c = state[2],
        d = state[3],
        x[16];
  
#if ENCODE_DECODE
  Decode (x, block, 64);
#else
  x[0 ] = block[0 ] | (block[1 ] << 8) | (block[2 ] << 16) | (block[3 ] << 24);
  x[1 ] = block[4 ] | (block[5 ] << 8) | (block[6 ] << 16) | (block[7 ] << 24);
  x[2 ] = block[8 ] | (block[9 ] << 8) | (block[10] << 16) | (block[11] << 24);
  x[3 ] = block[12] | (block[13] << 8) | (block[14] << 16) | (block[15] << 24);
  x[4 ] = block[16] | (block[17] << 8) | (block[18] << 16) | (block[19] << 24);
  x[5 ] = block[20] | (block[21] << 8) | (block[22] << 16) | (block[23] << 24);
  x[6 ] = block[24] | (block[25] << 8) | (block[26] << 16) | (block[27] << 24);
  x[7 ] = block[28] | (block[29] << 8) | (block[30] << 16) | (block[31] << 24);
  x[8 ] = block[32] | (block[33] << 8) | (block[34] << 16) | (block[35] << 24);
  x[9 ] = block[36] | (block[37] << 8) | (block[38] << 16) | (block[39] << 24);
  x[10] = block[40] | (block[41] << 8) | (block[42] << 16) | (block[43] << 24);
  x[11] = block[44] | (block[45] << 8) | (block[46] << 16) | (block[47] << 24);
  x[12] = block[48] | (block[49] << 8) | (block[50] << 16) | (block[51] << 24);
  x[13] = block[52] | (block[53] << 8) | (block[54] << 16) | (block[55] << 24);
  x[14] = block[56] | (block[57] << 8) | (block[58] << 16) | (block[59] << 24);
  x[15] = block[60] | (block[61] << 8) | (block[62] << 16) | (block[63] << 24);
#endif

  /* Round 1 */
  FF(a, b, c, d, x[ 0], S11, 0xd76aa478); /* 1 */
  FF(d, a, b, c, x[ 1], S12, 0xe8c7b756); /* 2 */
  FF(c, d, a, b, x[ 2], S13, 0x242070db); /* 3 */
  FF(b, c, d, a, x[ 3], S14, 0xc1bdceee); /* 4 */
  FF(a, b, c, d, x[ 4], S11, 0xf57c0faf); /* 5 */
  FF(d, a, b, c, x[ 5], S12, 0x4787c62a); /* 6 */
  FF(c, d, a, b, x[ 6], S13, 0xa8304613); /* 7 */
  FF(b, c, d, a, x[ 7], S14, 0xfd469501); /* 8 */
  FF(a, b, c, d, x[ 8], S11, 0x698098d8); /* 9 */
  FF(d, a, b, c, x[ 9], S12, 0x8b44f7af); /* 10 */
  FF(c, d, a, b, x[10], S13, 0xffff5bb1); /* 11 */
  FF(b, c, d, a, x[11], S14, 0x895cd7be); /* 12 */
  FF(a, b, c, d, x[12], S11, 0x6b901122); /* 13 */
  FF(d, a, b, c, x[13], S12, 0xfd987193); /* 14 */
  FF(c, d, a, b, x[14], S13, 0xa679438e); /* 15 */
  FF(b, c, d, a, x[15], S14, 0x49b40821); /* 16 */

  /* Round 2 */
  GG(a, b, c, d, x[ 1], S21, 0xf61e2562); /* 17 */
  GG(d, a, b, c, x[ 6], S22, 0xc040b340); /* 18 */
  GG(c, d, a, b, x[11], S23, 0x265e5a51); /* 19 */
  GG(b, c, d, a, x[ 0], S24, 0xe9b6c7aa); /* 20 */
  GG(a, b, c, d, x[ 5], S21, 0xd62f105d); /* 21 */
  GG(d, a, b, c, x[10], S22,  0x2441453); /* 22 */
  GG(c, d, a, b, x[15], S23, 0xd8a1e681); /* 23 */
  GG(b, c, d, a, x[ 4], S24, 0xe7d3fbc8); /* 24 */
  GG(a, b, c, d, x[ 9], S21, 0x21e1cde6); /* 25 */
  GG(d, a, b, c, x[14], S22, 0xc33707d6); /* 26 */
  GG(c, d, a, b, x[ 3], S23, 0xf4d50d87); /* 27 */
  GG(b, c, d, a, x[ 8], S24, 0x455a14ed); /* 28 */
  GG(a, b, c, d, x[13], S21, 0xa9e3e905); /* 29 */
  GG(d, a, b, c, x[ 2], S22, 0xfcefa3f8); /* 30 */
  GG(c, d, a, b, x[ 7], S23, 0x676f02d9); /* 31 */
  GG(b, c, d, a, x[12], S24, 0x8d2a4c8a); /* 32 */

  /* Round 3 */
  HH(a, b, c, d, x[ 5], S31, 0xfffa3942); /* 33 */
  HH(d, a, b, c, x[ 8], S32, 0x8771f681); /* 34 */
  HH(c, d, a, b, x[11], S33, 0x6d9d6122); /* 35 */
  HH(b, c, d, a, x[14], S34, 0xfde5380c); /* 36 */
  HH(a, b, c, d, x[ 1], S31, 0xa4beea44); /* 37 */
  HH(d, a, b, c, x[ 4], S32, 0x4bdecfa9); /* 38 */
  HH(c, d, a, b, x[ 7], S33, 0xf6bb4b60); /* 39 */
  HH(b, c, d, a, x[10], S34, 0xbebfbc70); /* 40 */
  HH(a, b, c, d, x[13], S31, 0x289b7ec6); /* 41 */
  HH(d, a, b, c, x[ 0], S32, 0xeaa127fa); /* 42 */
  HH(c, d, a, b, x[ 3], S33, 0xd4ef3085); /* 43 */
  HH(b, c, d, a, x[ 6], S34,  0x4881d05); /* 44 */
  HH(a, b, c, d, x[ 9], S31, 0xd9d4d039); /* 45 */
  HH(d, a, b, c, x[12], S32, 0xe6db99e5); /* 46 */
  HH(c, d, a, b, x[15], S33, 0x1fa27cf8); /* 47 */
  HH(b, c, d, a, x[ 2], S34, 0xc4ac5665); /* 48 */

  /* Round 4 */
  II(a, b, c, d, x[ 0], S41, 0xf4292244); /* 49 */
  II(d, a, b, c, x[ 7], S42, 0x432aff97); /* 50 */
  II(c, d, a, b, x[14], S43, 0xab9423a7); /* 51 */
  II(b, c, d, a, x[ 5], S44, 0xfc93a039); /* 52 */
  II(a, b, c, d, x[12], S41, 0x655b59c3); /* 53 */
  II(d, a, b, c, x[ 3], S42, 0x8f0ccc92); /* 54 */
  II(c, d, a, b, x[10], S43, 0xffeff47d); /* 55 */
  II(b, c, d, a, x[ 1], S44, 0x85845dd1); /* 56 */
  II(a, b, c, d, x[ 8], S41, 0x6fa87e4f); /* 57 */
  II(d, a, b, c, x[15], S42, 0xfe2ce6e0); /* 58 */
  II(c, d, a, b, x[ 6], S43, 0xa3014314); /* 59 */
  II(b, c, d, a, x[13], S44, 0x4e0811a1); /* 60 */
  II(a, b, c, d, x[ 4], S41, 0xf7537e82); /* 61 */
  II(d, a, b, c, x[11], S42, 0xbd3af235); /* 62 */
  II(c, d, a, b, x[ 2], S43, 0x2ad7d2bb); /* 63 */
  II(b, c, d, a, x[ 9], S44, 0xeb86d391); /* 64 */

  state[0] += a;
  state[1] += b;
  state[2] += c;
  state[3] += d;

/* secure */
  memset((uns8*)&x, 0, sizeof(x));
}

#if ENCODE_DECODE
/* Encodes input (uns32) into output (uns8). Assumes len is
     a multiple of 4.
 */
void Encode(uns8 *output, uns32 *input, uns32 len) {
  int i, j;

  for (i=0,j=0; j<len; i++,j+=4) {
    output[j] = (uns8)(input[i] & 0xff);
    output[j+1] = (uns8)((input[i] >> 8) & 0xff);
    output[j+2] = (uns8)((input[i] >> 16) & 0xff);
    output[j+3] = (uns8)((input[i] >> 24) & 0xff);
  }
}

/* Decodes input (uns8) into output (uns32). Assumes len is
     a multiple of 4.
 */
void Decode(uns32 *output, uns8 *input, uns32 len) {
  int i,j;

  for (i=0,j=0; j<len; i++,j+=4)
    output[i] = input[j] | (input[j+1] << 8) | (input[j+2] << 16) | (input[j+3] << 24);
}
#endif

#endif
